package org.kantega.jexmec.ctor;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.List;

/**
 * Locates classes by reading their class names from files found on the classpath.
 * The files are expected to have one fully qualified class name on each line.
 */
public class ClassLocator<T extends Class> {
    private final String[] servicesFiles;

    /**
     * Read class names from the given class path locations.
     * @param servicesFiles locations to look for class lists
     */
    public ClassLocator(String... servicesFiles) {
        this.servicesFiles = new String[servicesFiles.length];
        System.arraycopy(servicesFiles, 0, this.servicesFiles, 0, servicesFiles.length);
    }


    /**
     * Convenience constructor reading files from the given file name in a directory determined by the name of the plugin class.
     * If the plugin class is named <code>com.example.pluginapi.MyPlugin</code> and the filename is "simple", the class
     * loactor will look in the following two locations:
     * <ul>
     * <li><code>META-INF/services/MyPlugin/filename</code></li>
     * <li><code>META-INF/services/com.example.pluginapi.MyPlugin/filename</code></li>  
     * </ul>
     * @param pluginClass the plugin class to locate classes for
     * @param filename the file name within <code>META-INF/services/MyPlugin/</code> to read
     */
    public ClassLocator(Class pluginClass, String filename) {
        this("META-INF/services/" +pluginClass.getName() +"/" + filename,
                "META-INF/services/" +pluginClass.getSimpleName() +"/" + filename);
    }
    
    public List<T> locateClasses(ClassLoader classLoader) {
        List<T> classes = new ArrayList<T>();

        try {
            for(String serviceFile : servicesFiles) {
                Enumeration<URL> resources = classLoader.getResources(serviceFile);
                while (resources.hasMoreElements()) {
                    final URL url = resources.nextElement();

                    InputStream inputStream = null;
                    InputStreamReader inputStreamReader = null;
                    BufferedReader bufferedReader = null;

                    try {
                        inputStream = url.openStream();
                        inputStreamReader = new InputStreamReader(inputStream, "utf-8");
                        bufferedReader = new BufferedReader(inputStreamReader);

                        String className;
                        while ((className = bufferedReader.readLine()) != null) {
                            className = className.trim();
                            if (className.length() > 0) {

                                try {
                                    @SuppressWarnings("unchecked")
                                    T clazz = (T) classLoader.loadClass(className);
                                    classes.add(clazz);
                                } catch (ClassNotFoundException e) {
                                    throw new InvalidPluginException("Plugin descriptor " +url +" specifies a class that can't be found: " + className, e, null);
                                }

                            }
                        }
                    } finally {
                        if (bufferedReader != null) {
                            bufferedReader.close();
                        }
                        if (inputStreamReader != null) {
                            inputStreamReader.close();
                        }
                        if (inputStream != null) {
                            inputStream.close();
                        }
                    }
                }
            }
            return classes;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
